package com.qxgmat.service;

import com.github.pagehelper.Page;
import com.nuliji.tools.AbstractService;
import com.nuliji.tools.Transform;
import com.nuliji.tools.exception.ParameterException;
import com.nuliji.tools.exception.SystemException;
import com.nuliji.tools.mybatis.Example;
import com.qxgmat.data.constants.enums.status.HomeworkPreviewStatus;
import com.qxgmat.data.dao.HomeworkPreviewMapper;
import com.qxgmat.data.dao.entity.HomeworkPreview;
import com.qxgmat.data.dao.entity.UserPaper;
import com.qxgmat.data.dao.entity.UserReport;
import com.qxgmat.data.relation.UserPaperRelationMapper;
import com.qxgmat.data.relation.UserReportRelationMapper;
import com.qxgmat.data.relation.entity.UserHomeworkPreviewRelation;
import com.qxgmat.service.inline.UserPaperService;
import com.qxgmat.service.inline.UserReportService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.Date;
import java.util.List;

@Service
public class HomeworkPreviewService extends AbstractService {
    private static final Logger logger = LoggerFactory.getLogger(HomeworkPreviewService.class);

    @Resource
    private HomeworkPreviewMapper homeworkPreviewMapper;

    @Resource
    private UserReportService userReportService;

    @Resource
    private UserPaperService userPaperService;

    @Resource
    private UserReportRelationMapper userReportRelationMapper;

    @Resource
    private UserPaperRelationMapper userPaperRelationMapper;

    /**
     * 查找用户作业，并关联用户完成度最高的最后一次
     * @param page
     * @param size
     * @param category
     * @param userId
     * @param previewId
     * @param startTime
     * @param endTime
     * @return
     */
    public Page<UserHomeworkPreviewRelation> listAdmin(int page, int size, Number category, Number userId, Number previewId, String startTime, String endTime){
        Page<UserHomeworkPreviewRelation> p = page(() -> {
            userPaperRelationMapper.listHomeworkPreviewAdmin(category, userId, previewId, startTime, endTime);
        }, page, size);

        Collection ids = Transform.getIds(p, UserReport.class, "id");

        List<UserPaper> list = userPaperService.select(ids);
        Transform.replace(p, Transform.convert(list, UserHomeworkPreviewRelation.class), UserHomeworkPreviewRelation.class, "id");

        List<UserReport> reportList = userReportRelationMapper.listFinishLast(ids, startTime, endTime);
        Collection reportIds = Transform.getIds(reportList, UserReport.class, "id");
        Transform.replace(reportList, userReportService.select(reportIds), UserReport.class, "id");

        Transform.combine(p, reportList, UserHomeworkPreviewRelation.class, "id", "report", UserReport.class, "paperId");

        return p;
    }

    /**
     * 查找用户作业, 并关联用户最后一次作业
     * @param page
     * @param size
     * @param category
     * @param userId
     * @param endTime
     * @param finish
     * @return
     */
    public Page<UserHomeworkPreviewRelation> list(int page, int size, Number category, Number userId, String endTime, Boolean finish){
        Page<UserHomeworkPreviewRelation> p = page(()->{
            userPaperRelationMapper.listHomeworkPreview(category, userId, endTime, finish);
        },page, size);

        Collection ids = Transform.getIds(p, UserReport.class, "id");

        List<UserPaper> list = userPaperService.select(ids);
        Transform.replace(p, Transform.convert(list, UserHomeworkPreviewRelation.class), UserHomeworkPreviewRelation.class, "id");

        List<UserReport> reportList = userReportRelationMapper.listLast(ids);
        Collection reportIds = Transform.getIds(reportList, UserReport.class, "id");
        Transform.replace(reportList, userReportService.select(reportIds), UserReport.class, "id");

        Transform.combine(p, reportList, UserHomeworkPreviewRelation.class, "id", "paperId", UserReport.class, "paperId");

        return p;
    }

    /**
     * 完成预习作业
     * @param report
     */
    public void Finish(UserReport report){

    }

    @Transactional
    public HomeworkPreview add(HomeworkPreview homeworkPreview){
        int result = insert(homeworkPreviewMapper, homeworkPreview);
        homeworkPreview = one(homeworkPreviewMapper, homeworkPreview.getId());
        if(homeworkPreview == null){
            throw new SystemException("预习作业添加失败");
        }
        // todo 添加所有用户预习作业

        return homeworkPreview;
    }

    @Transactional
    public HomeworkPreview edit(HomeworkPreview homeworkPreview){
        HomeworkPreview in = one(homeworkPreviewMapper, homeworkPreview.getId());
        if(in == null){
            throw new ParameterException("预习作业不存在");
        }
        // todo 修改所有用户预习作业
        int result = update(homeworkPreviewMapper, homeworkPreview);
        return homeworkPreview;
    }

    public boolean delete(Number id){
        HomeworkPreview in = one(homeworkPreviewMapper, id);
        if(in == null){
            throw new ParameterException("预习作业不存在");
        }
        int result = delete(homeworkPreviewMapper, id);
        return result > 0;
    }

    public HomeworkPreview get(Number id){
        HomeworkPreview in = one(homeworkPreviewMapper, id);

        if(in == null){
            throw new ParameterException("预习作业不存在");
        }
        return in;
    }

    public Page<HomeworkPreview> select(int page, int pageSize){
        return select(homeworkPreviewMapper, page, pageSize);
    }

    public Page<HomeworkPreview> select(int page, int pageSize, Integer category, HomeworkPreviewStatus status){
        Example example = new Example(HomeworkPreview.class);
        if(category != null && category>0)
            example.and(
                    example.createCriteria().andEqualTo("category", category)
            );
        if (status!=HomeworkPreviewStatus.ALL){
            tk.mybatis.mapper.entity.Example.Criteria criteria = example.createCriteria();
            switch(status){
                case NO_START:
                    criteria.andLessThan("startTime", new Date());
                    break;
                case PROCESS:
                    criteria.andGreaterThanOrEqualTo("startTime", new Date()).andLessThan("endTime", new Date());
                    break;
                case FINISH:
                    criteria.andGreaterThanOrEqualTo("endTime", new Date());
                    break;
            }
            example.and(criteria);
        }
        example.orderBy("id").desc();
        return select(homeworkPreviewMapper, example, page, pageSize);
    }

    public List<HomeworkPreview> select(Collection ids){
        return select(homeworkPreviewMapper, ids);
    }

}
