package com.qxgmat.service.inline;

import com.github.pagehelper.Page;
import com.nuliji.tools.AbstractService;
import com.nuliji.tools.exception.ParameterException;
import com.nuliji.tools.exception.SystemException;
import com.qxgmat.data.constants.enums.status.DirectionStatus;
import com.qxgmat.data.dao.ExerciseQuestionMapper;
import com.qxgmat.data.dao.entity.ExerciseQuestion;
import com.qxgmat.data.relation.ExerciseQuestionRelationMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class ExerciseQuestionService extends AbstractService {
    private static final Logger logger = LoggerFactory.getLogger(ExerciseQuestionService.class);

    @Resource
    private ExerciseQuestionMapper exerciseQuestionMapper;

    @Resource
    private ExerciseQuestionRelationMapper exerciseQuestionRelationMapper;


    public Page<ExerciseQuestion> listAdmin(int page, int pageSize, Number structId, Number questionNo, Number paperId, String place, String difficult, String startTime, String endTime, String order, DirectionStatus direction){
        if(order.isEmpty()) order = "id";
        if (direction == null){
            direction = DirectionStatus.DESC;
        }
        String finalOrder = order;
        DirectionStatus finalDirection = direction;

        return page(() -> {
            exerciseQuestionRelationMapper.listAdmin(structId, questionNo, paperId, place, difficult, startTime, endTime, finalOrder, finalDirection.key);
        }, page, pageSize);
    }

    public ExerciseQuestion add(ExerciseQuestion question){
        int result = insert(exerciseQuestionMapper, question);
        question = one(exerciseQuestionMapper, question.getId());
        if(question == null){
            throw new SystemException("题目添加失败");
        }
        return question;
    }

    public ExerciseQuestion edit(ExerciseQuestion question){
        ExerciseQuestion in = one(exerciseQuestionMapper, question.getId());
        if(in == null){
            throw new ParameterException("题目不存在");
        }
        int result = update(exerciseQuestionMapper, question);
        return question;
    }

    public boolean delete(Number id){
        ExerciseQuestion in = one(exerciseQuestionMapper, id);
        if(in == null){
            throw new ParameterException("题目不存在");
        }
        int result = delete(exerciseQuestionMapper, id);
        return result > 0;
    }

    public ExerciseQuestion get(Number id){
        ExerciseQuestion in = one(exerciseQuestionMapper, id);

        if(in == null){
            throw new ParameterException("题目不存在");
        }
        return in;
    }

    public Page<ExerciseQuestion> select(int page, int pageSize){
        return select(exerciseQuestionMapper, page, pageSize);
    }

    public List<ExerciseQuestion> select(Collection ids){
        return select(exerciseQuestionMapper, ids);
    }

}
